from ctypes import *
import ctypes
from ctypes import WinDLL
from ctypes.wintypes import *

EnumWindows = ctypes.windll.user32.EnumWindows
EnumWindowsProc = ctypes.WINFUNCTYPE(
    # What was (wrongly?) in the SO post: https://stackoverflow.com/questions/14653168/get-hwnd-of-each-window
    # ctypes.c_bool, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int)
    ctypes.c_bool,
    HWND,
    ctypes.POINTER(ctypes.c_int),
)
GetWindowText = ctypes.windll.user32.GetWindowTextW
GetWindowTextLength = ctypes.windll.user32.GetWindowTextLengthW
IsWindowVisible = ctypes.windll.user32.IsWindowVisible


def _find_hwnd(title):
    """Find HWND by title. Alternate way to get a window handle."""
    windows = []

    def foreach_window(hwnd, lParam):
        if IsWindowVisible(hwnd):
            length = GetWindowTextLength(hwnd)
            buff = ctypes.create_unicode_buffer(length + 1)
            GetWindowText(hwnd, buff, length + 1)
            title = buff.value
            if title:
                windows.append((hwnd, title))
        return True

    EnumWindows(EnumWindowsProc(foreach_window), 0)

    found = None
    for w in windows:
        if w[1] == title:
            found = w
    return found[0]


def _find_hwnd_test_callback():
    print("Looking for the window...")
    hwnd = _find_hwnd("pysetwindowpos")
    print("found hwnd=", hwnd)
    print("frame()=", _find_hwnd_test_root.frame())
    print("int(frame(), 16)=", int(_find_hwnd_test_root.frame(), 16))
    print("winfo_id()=", _find_hwnd_test_root.winfo_id())


def _find_hwnd_test():

    import tkinter as tk

    global _find_hwnd_test_root
    _find_hwnd_test_root = root = tk.Tk()
    root.title("pysetwindowpos")
    root.geometry("200x100")
    a = tk.Label(root, text="I'm on top of the world!")
    a.pack()

    # Need to wait until main loop starts (i.e., window actually displayed)
    # to find window. Doesn't work with `0` as first parameter.
    root.after(1, _find_hwnd_test_callback)

    print("Starting tkinter main loop...")
    root.mainloop()


if __name__ == "__main__":
    _find_hwnd_test()



from ctypes import *
import ctypes
from ctypes.wintypes import HWND, UINT

# To publish:
# TODO: only export what I actually want to?
# TODO: package for publishing
# TODO: publish on pypi


def set_window_topmost(hwnd):
    HWND_TOPMOST = -1
    SWP_NOSIZE = 0x0001
    SWP_NOMOVE = 0x0002
    # TODO: `ctypes.windll.user32` - why didn't that work? (and then how to set use_last_error?)
    user32 = WinDLL("user32", use_last_error=True)
    SetWindowPos = user32.SetWindowPos
    SetWindowPos.argtypes = [HWND, HWND, c_int, c_int, c_int, c_int, UINT]
    SetWindowPos.restype = c_int
    SetWindowPos.errcheck = _errcheck

    SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE)


def _errcheck(result, func, args):
    if not result:
        raise WinError(get_last_error())


if __name__ == "__main__":

    import tkinter as tk

    root = tk.Tk()
    root.title("pysetwindowpos")
    root.geometry("200x100")
    a = tk.Label(root, text="I'm on top of the world!")
    a.pack()

    def position_window():
        hwnd_frame = int(root.frame(), 16)  # cf. than winfo_id ?
        print("hwnd_frame=", hwnd_frame)
        set_window_topmost(hwnd_frame)

    # Need to wait until main loop starts (i.e., window actually displayed)
    # to find/control window. Doesn't work with `0` as first parameter.
    root.after(1, position_window)

    root.mainloop()
